//  Main.cpp version 1.0
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998  Gaspar Sinai
// 
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
// This is the example program for the unicode library
// Gaspar Sinai <gsinai@gol.com>
//

#include <X11/Xlib.h>

#include "Main.h"
#include "MainWidget.h"
#include <stdlib.h>

#ifdef X_LOCALE
# ifdef LC_ALL
#  undef LC_ALL
# endif
# ifdef LC_COLLATE
#  undef LC_COLLATE
# endif
# ifdef LC_CTYPE
#  undef LC_CTYPE
# endif
# ifdef LC_MONETARY
#  undef LC_MONETARY
# endif
# ifdef LC_NUMERIC
#  undef LC_NUMERIC
# endif
# ifdef LC_TIME
#  undef LC_TIME
# endif
#endif /* X_LOCALE */

#include <X11/Xlocale.h>

static Config* lConfig;

int
main (int argc, char *argv[])
{
	YApplication		*app;
	MainWidget		*widget;
	int			i;
	int			j;
	int			k;
	const FontMap*		fontMap;
	const TrueTypeFont*	trueTypeFont;
	const char*		umap;
	const char*		foundry;
	const char*		family;
	const char*		registry;
	const char*		encoding;
	UFreeType*		xFreeType;
	const char*		xFileName;
	Config			mainConfig ("yuditrc");
	
	// It may dump if I miss it.
	putenv ("LANG=");
	putenv ("LANGUAGE=");
	lConfig = &mainConfig;

#ifdef WITH_KDE
	app = new YApplication (argc, argv, MAINPRG);
#else
	app = new YApplication (argc, argv);
#endif

#ifdef USE_SETLOCALE
	setlocale (LC_ALL, "");
	XSetLocaleModifiers ("");
#endif

#ifdef WITH_KDE
	// This produces reaaly big core files on DEC Alpha.
	// in glibc-2.0.6 textdomain.c:
	// 86       char *cp = (char *) malloc (len);
	// len is 7
	//InitTranslate (kapp->appName().data(), kapp->kdedir() + "/locale");
#endif
	if (mainConfig.getConfigPath().isNull())
	{
		QMessageBox::message (Translate ("Yudit: Info"),
			Translate ("Copying system config file."),
			Translate ("OK"));
		mainConfig.copySysConfig("yuditrc");
		if (mainConfig.getErrors () != 0)
		{
			QMessageBox::message (Translate ("Yudit: Error"),
			 mainConfig.getErrors (),
			 Translate ("OK"));
			exit (1);
		}
		mainConfig.readConfig ("yuditrc");
	}
	if (mainConfig.getErrors () != 0)
	{
		QMessageBox::message (Translate ("Yudit: Error"), mainConfig.getErrors(),
			Translate ("OK"));
#ifdef WITH_KDE
		kapp->quit();
#endif
		exit (0);
	}

	USetMapPath ((const char*) mainConfig.getPath ());
	//
	// Load the fontMaps
	//
	for (i=0; i<mainConfig.getCheckList (Config::FONTNAME).getSize(); i++)
	{
		fontMap = mainConfig.getFontMap (
			mainConfig.getCheckList (Config::FONTNAME).at(i));
		if (fontMap!=0) for (j=0; j<fontMap->size; j++)
		{
			foundry = fontMap->list[FontMap::FOUNDRY].at (j);
			family = fontMap->list[FontMap::FAMILY].at (j);
			registry = fontMap->list[FontMap::REGISTRY].at (j);
			encoding = fontMap->list[FontMap::ENCODING].at (j);
			umap = fontMap->list[FontMap::UMAP].at (j);
			
			UAddFontMap ((const char*) fontMap->name,
				foundry, family, registry, encoding, umap);
		}
		if (fontMap==0)
		{
			// It is checked in config load.
			trueTypeFont = mainConfig.getTrueTypeFont(
				mainConfig.getCheckList (Config::FONTNAME).at(i));
			if (trueTypeFont!=0)
			{
				UAddFreeType ((const char*)trueTypeFont->name);
				xFreeType = UGetFreeTypePrint (
					(const char*) trueTypeFont->name);
				if (xFreeType==0)
				{
					cerr << "error: count not add TrueType font '" 
					<< (const char*)trueTypeFont->name
					<< "'.\n";
					continue;
				}
				for (j=0; j<UFreeType::WEIGHT_COUNT; j++)
				{
				 for (k=0; k<UFreeType::SLANT_COUNT; k++)
				 {
					xFileName = (const char*)trueTypeFont->fontFile[j][k];
					xFreeType->setFontFile (
						xFileName,
						(UFreeType::UWeight) j,
						(UFreeType::USlant) k);
				 }
				}
			}
		}
	}

	if (argc < 2)
	{
		widget = new MainWidget (mainConfig, ((argc > 0) ? argv[1] : 0));
	}
	else
	{
		widget = new MainWidget (mainConfig, argv[1], argv[2]);
	}
	return (app->exec());
}

